'use strict';

var chalk = require('chalk');
var diff = require('jest-diff');
var stripAnsi = require('strip-ansi');

var _require = require('jest-snapshot'),
    toMatchSnapshot = _require.toMatchSnapshot;

var isAddition = function isAddition(line) {
  return (/^\+/.test(line)
  );
};

var isDeletion = function isDeletion(line) {
  return (/^-/.test(line)
  );
};

var isClassNameAttribute = function isClassNameAttribute(line) {
  return (/class(Name)?="css-.*"/.test(line)
  );
};
var isDataAttribute = function isDataAttribute(line) {
  return (/data-css-.*/.test(line)
  );
};
var isClassNameSelector = function isClassNameSelector(line) {
  return (/\.css-.*,/.test(line)
  );
};
var isDataSelector = function isDataSelector(line) {
  return (/\[data-css-.*\] {/.test(line)
  );
};

var isClassName = function isClassName(line) {
  return (isAddition(line) || isDeletion(line)) && (isClassNameAttribute(line) || isDataAttribute(line) || isClassNameSelector(line) || isDataSelector(line));
};

var colorize = function colorize(message) {
  return message.split('\n').map(function (line) {
    if (isClassName(line)) {
      return chalk.white(line);
    }
    if (isAddition(line)) {
      return chalk.red(line);
    }
    if (isDeletion(line)) {
      return chalk.green(line);
    }
    return chalk.dim(line);
  }).join('\n');
};

var matcher = {
  toMatchSnapshotWithGlamor() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var result = toMatchSnapshot.apply(this, args);
    var message = void 0;

    if (!result.pass) {
      message = diff(result.expected, result.actual, {
        aAnnotation: 'Snapshot',
        bAnnotation: 'Received'
      });
      message = stripAnsi(message);
      message = colorize(message);
    }

    return { pass: result.pass, message };
  }
};

module.exports = matcher;