'use strict';

var css = require('css');

var _require = require('glamor'),
    styleSheet = _require.styleSheet;

var serializer = { test, print };

module.exports = serializer;

function test(val) {
  return !val.withStyles && val.$$typeof === Symbol.for('react.test.json');
}

function print(val, printer) {
  var selectors = getSelectors(val);
  var styles = getStyles(selectors);
  val.withStyles = true;
  var printedVal = printer(val);
  if (styles) {
    return `${styles}\n\n${printedVal}`;
  } else {
    return printedVal;
  }
}

function getSelectors(node) {
  var selectors = [];
  if (node.children && node.children.reduce) {
    selectors = node.children.reduce(function (acc, child) {
      return acc.concat(getSelectors(child));
    }, []);
  }
  if (node.props) {
    return getSelectorsFromProps(selectors, node.props);
  }
  return selectors;
}

function getSelectorsFromProps(selectors, props) {
  var className = props.className || props.class;
  if (className) {
    selectors = selectors.concat(className.toString().split(' ').map(function (cn) {
      return `.${cn}`;
    }));
  }
  var dataProps = Object.keys(props).reduce(function (dProps, key) {
    if (key.startsWith('data-')) {
      dProps.push(`[${key}]`);
    }
    return dProps;
  }, []);
  if (dataProps.length) {
    selectors = selectors.concat(dataProps);
  }
  return selectors;
}

function getStyles(nodeSelectors) {
  var styles = styleSheet.tags.map(function (tag) {
    return (/* istanbul ignore next */tag.textContent || ''
    );
  }).join('\n');
  var ast = css.parse(styles);
  ast.stylesheet.rules = ast.stylesheet.rules.filter(function (rule) {
    return rule.type === 'rule' && rule.selectors.some(function (selector) {
      return nodeSelectors.includes(selector);
    });
  });

  var ret = css.stringify(ast);
  return ret;
}